"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const models_1 = require("../models");
const logger_service_1 = require("../services/logger.service");
const database_service_1 = require("../services/database.service");
const exceptions_1 = require("../exceptions");
const decorators_1 = require("../decorators");
const pipes_1 = require("../pipes");
const rxjs_1 = require("rxjs");
let DatabaseController = class DatabaseController {
    constructor(databaseSvc, loggerEnabled) {
        this.databaseSvc = databaseSvc;
        this.logger = new logger_service_1.Logger('Database', loggerEnabled);
    }
    find(quieries, user, tableName, index) {
        this.logger.log(`Find get`, tableName);
        if (user === undefined) {
            const validateTable = (__) => tableName === __;
            if (!validateTable('Users') && !validateTable('Organizations')) {
                throw new exceptions_1.UnauthorizedException('You are not authorized to access this resource.');
            }
        }
        return this.databaseSvc.find(quieries, { name: tableName, index }, user);
    }
    batchGet(batchInput) {
        this.logger.log(`Batch get`, batchInput);
        return this.databaseSvc.batchGet(batchInput.ids, batchInput.collections);
    }
    slice(quieries, tableName, indexName) {
        this.logger.log(`Slice get`, tableName);
        return this.databaseSvc.slice(quieries, { name: tableName, index: indexName });
    }
};
__decorate([
    common_1.Post('find'),
    __param(0, common_1.Body()),
    __param(1, decorators_1.User('public', pipes_1.UserPipe)), __param(2, common_1.Query('tableName')),
    __param(3, common_1.Query('index')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, models_1.UserDTO, String, String]),
    __metadata("design:returntype", rxjs_1.Observable)
], DatabaseController.prototype, "find", null);
__decorate([
    common_1.Post('batch'),
    __param(0, common_1.Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [models_1.DatabaseBatchDTO]),
    __metadata("design:returntype", rxjs_1.Observable)
], DatabaseController.prototype, "batchGet", null);
__decorate([
    common_1.Post('slice'),
    __param(0, common_1.Body()), __param(1, common_1.Query('tableName')),
    __param(2, common_1.Query('index')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, String, String]),
    __metadata("design:returntype", rxjs_1.Observable)
], DatabaseController.prototype, "slice", null);
DatabaseController = __decorate([
    common_1.Controller('database'),
    common_1.UseInterceptors(exceptions_1.ErrorsInterceptor),
    common_1.UseFilters(exceptions_1.ErrorFilter),
    __param(1, common_1.Inject('loggerEnabled')), __param(1, common_1.Optional()),
    __metadata("design:paramtypes", [database_service_1.ServerDatabaseService, Boolean])
], DatabaseController);
exports.DatabaseController = DatabaseController;
