"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const resource_service_1 = require("../services/resource.service");
const rxjs_1 = require("rxjs");
const models_1 = require("../models");
const logger_service_1 = require("../services/logger.service");
const decorators_1 = require("../decorators");
const pipes_1 = require("../pipes");
const exceptions_1 = require("../exceptions");
const operators_1 = require("rxjs/operators");
let ResourceController = class ResourceController {
    constructor(resourceSvc, loggerEnabled) {
        this.resourceSvc = resourceSvc;
        this.logger = new logger_service_1.Logger('Resource Controller', loggerEnabled);
    }
    getAll(user) {
        this.logger.log(`Get resource tree`, user);
        return this.resourceSvc.getAll(user).pipe(operators_1.catchError(error => {
            this.logger.error(error);
            return rxjs_1.throwError(error);
        }));
    }
    get(id, user, tableName, indexName, attribute) {
        if (!id) {
            return this.getAll(user);
        }
        this.logger.log(`Get resource (${id})`);
        return this.resourceSvc.get(id, user, { name: tableName, index: indexName }, attribute);
    }
    save(dataDTO, user, tableName, merge = true) {
        if (typeof merge === 'string') {
            merge = merge !== 'false';
        }
        if (user === undefined) {
            const validateTable = (__) => dataDTO.tableName === __ || tableName === __;
            if (!validateTable('Users') && !validateTable('Organizations')) {
                throw new exceptions_1.UnauthorizedException('You are not authorized to access this resource.');
            }
        }
        if (tableName) {
            const resourced = Object.assign(dataDTO, { tableName });
            this.logger.log(`Save resource`, resourced);
            return this.resourceSvc.save(resourced, user, merge);
        }
        else {
            this.logger.log(`Save resource`, dataDTO);
            return this.resourceSvc.save(dataDTO, user, merge);
        }
    }
};
__decorate([
    common_1.Get(),
    __param(0, decorators_1.User(pipes_1.UserPipe)),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [models_1.UserDTO]),
    __metadata("design:returntype", void 0)
], ResourceController.prototype, "getAll", null);
__decorate([
    common_1.Get(':id'),
    __param(0, common_1.Param('id')),
    __param(1, decorators_1.User(pipes_1.UserPipe)),
    __param(2, common_1.Query('tableName')),
    __param(3, common_1.Query('index')),
    __param(4, common_1.Query('attribute')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, models_1.UserDTO, String, String, String]),
    __metadata("design:returntype", rxjs_1.Observable)
], ResourceController.prototype, "get", null);
__decorate([
    common_1.Post(),
    __param(0, common_1.Body()),
    __param(1, decorators_1.User('public', pipes_1.UserPipe)),
    __param(2, common_1.Query('tableName')),
    __param(3, common_1.Query('merge')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [models_1.ResourceDTO,
        models_1.UserDTO, String, Boolean]),
    __metadata("design:returntype", rxjs_1.Observable)
], ResourceController.prototype, "save", null);
ResourceController = __decorate([
    common_1.Controller('resource'),
    common_1.UseInterceptors(exceptions_1.ErrorsInterceptor),
    common_1.UseFilters(exceptions_1.ErrorFilter),
    __param(1, common_1.Inject('loggerEnabled')), __param(1, common_1.Optional()),
    __metadata("design:paramtypes", [resource_service_1.ResourceService, Boolean])
], ResourceController);
exports.ResourceController = ResourceController;
