"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const common_1 = require("@nestjs/common");
const logger_service_1 = require("../services/logger.service");
const exceptions_1 = require("../exceptions");
const models_1 = require("../models");
const _3a_common_1 = require("3a-common");
const database_1 = require("3a-common/dist/database");
let DynamoService = class DynamoService {
    constructor(AWS, loggerEnabled) {
        this.logger = new logger_service_1.Logger('Dynamo Service', loggerEnabled);
        this.dynamo = new AWS.DynamoDB.DocumentClient({ apiVersion: '2012-08-10' });
    }
    get(id, collection, attribute) {
        if (id.id) {
            id = id.sub;
        }
        if (id === undefined) {
            return rxjs_1.of(undefined);
        }
        if (collection.index) {
            const queryParams = {
                TableName: collection.name,
                IndexName: collection.index,
                KeyConditionExpression: '#key = :key',
                ExpressionAttributeNames: {
                    '#key': attribute || 'id',
                },
                ExpressionAttributeValues: {
                    ':key': id,
                },
            };
            return rxjs_1.from(this.dynamo.query(queryParams).promise().catch(error => {
                this.logger.log('[Get] Internal Error', error);
                throw new exceptions_1.NotFoundException('No item was found in selected table.');
            })).pipe(operators_1.map(queryData => {
                this.logger.log('[Get] Get data returned');
                if (!queryData || queryData.$response.error || !queryData.Items) {
                    this.logger.log('[Get] Not found', { id, collection });
                    throw new exceptions_1.NotFoundException('No item was found.');
                }
                return queryData.Items[0];
            }));
        }
        else {
            const getParams = {
                TableName: collection.name,
                Key: {
                    id
                }
            };
            this.logger.log(JSON.stringify(getParams));
            return rxjs_1.from(this.dynamo.get(getParams).promise().catch(error => {
                this.logger.log('[Get] Internal Error', error);
                throw new exceptions_1.NotFoundException('Could not load Database data');
            })).pipe(operators_1.map(data => {
                if (data.$response.error || !data.Item) {
                    this.logger.error('[Get] Error retreiving data', data.$response.error);
                    rxjs_1.throwError(new exceptions_1.NotFoundException(`Could not get data: (${collection.name}) ${id}.`));
                }
                else {
                    return data.Item;
                }
            }));
        }
    }
    save(data, collection, merge = true) {
        data = Object.keys(data)
            .filter(key => data[key] !== undefined && data[key] !== '' && data[key] != null)
            .map(key => {
            const _ = {};
            _[key] = data[key];
            return _;
        }).reduce((complete, partial) => Object.assign(complete, partial), {});
        const resourceParams = {
            TableName: collection.name,
            Item: data,
            ReturnValues: 'ALL_OLD'
        };
        const errorCallback = error => {
            this.logger.error('[Save] Could not save data', { error, data });
            return {
                $response: {
                    error,
                },
                Attributes: undefined,
            };
        };
        return rxjs_1.from(this.dynamo.put(resourceParams).promise().catch(errorCallback)).pipe(operators_1.switchMap(savedData => {
            if (savedData.$response.error) {
                if (savedData.$response.error.retryable) {
                    return rxjs_1.timer(1000).pipe(operators_1.switchMap(() => this.save(data, collection, merge)));
                }
                else {
                    this.logger.error('[Save] Error saving data', savedData.$response.error);
                    throw new exceptions_1.BadRequestException(`Could not save data.`, data);
                }
            }
            else {
                if (savedData.Attributes && merge) {
                    const mergedData = Object.assign(savedData.Attributes, data);
                    const finalCallback = finalSavedData => {
                        if (finalSavedData.$response.error) {
                            if (finalSavedData.$response.error.retryable) {
                                return rxjs_1.timer(5000).pipe(operators_1.switchMap(() => rxjs_1.from(this.dynamo.put({
                                    TableName: collection.name,
                                    Item: mergedData
                                }).promise().catch(errorCallback)).pipe(operators_1.switchMap(finalCallback))));
                            }
                            else {
                                this.logger.error('[Save] Error saving data', finalSavedData.$response.error);
                                throw new exceptions_1.BadRequestException(`Could not save data.`, mergedData);
                            }
                        }
                        else {
                            return rxjs_1.of(mergedData);
                        }
                    };
                    return rxjs_1.from(this.dynamo.put({
                        TableName: collection.name,
                        Item: mergedData
                    }).promise().catch(errorCallback)).pipe(operators_1.switchMap(finalCallback));
                }
                else {
                    return rxjs_1.of(data);
                }
            }
        }));
    }
    batchGet(ids, collections) {
        if (!ids || ids.length === 0) {
            this.logger.error('[Batch Get] Empty IDs array', ids);
            throw new exceptions_1.BadRequestException('Can\'t handle empty id array in batch get.');
        }
        if (collections.every(collection => collection.index === undefined)) {
            const batchParams = {
                RequestItems: {}
            };
            const tableNames = collections ? collections.map(col => col.name) : [models_1.ResourceTable];
            if (tableNames.length !== ids.length) {
                this.logger.error('[Batch Get] Dimmension mismatch', { ids, tableNames });
                throw new exceptions_1.BadRequestException('Batch input dimension mismatch. Datasets requested must be complementary.', { ids, tableNames });
            }
            batchParams.RequestItems = ids.map((idList, index) => {
                const batchMap = {};
                batchMap[tableNames[index]] = {
                    Keys: idList.map(id => {
                        return {
                            id
                        };
                    })
                };
                return batchMap;
            }).reduce((request, batchMap) => Object.assign(request, batchMap), {});
            return rxjs_1.from(this.dynamo.batchGet(batchParams).promise().catch(error => {
                this.logger.log('[Batch] Internal Error', error);
                throw new exceptions_1.NotFoundException(`Requested items not found: (${tableNames}) ${ids}`);
            })).pipe(operators_1.map(batchData => {
                if (batchData.$response.error || !batchData.Responses) {
                    this.logger.log('[Batch Get] Not found', { ids, tableNames });
                    throw new exceptions_1.NotFoundException(`Requested items not found: (${tableNames}) ${ids}`);
                }
                return batchData.Responses;
            }));
        }
        else if (collections.every(collection => !!collection.index)) {
            return rxjs_1.combineLatest(collections.map((collection, index) => {
                const EAV = {};
                const scanParams = {
                    TableName: collection.name,
                    IndexName: collection.index,
                    FilterExpression: ids[index].map((key, indexKey) => {
                        EAV[`:${indexKey}`] = key;
                        return `#id = :${indexKey}`;
                    }).join(' OR '),
                    ExpressionAttributeNames: { '#id': 'id' },
                };
                scanParams.ExpressionAttributeValues = EAV;
                return rxjs_1.from(this.dynamo.scan(scanParams).promise().catch(error => {
                    this.logger.log('[Get] Internal Error', error);
                    throw new exceptions_1.NotFoundException('No item was found in selected table.');
                })).pipe(operators_1.map(queryData => {
                    this.logger.log('[Get] Get data returned');
                    if (!queryData || queryData.$response.error || !queryData.Items) {
                        this.logger.log('[Get] Not found', { ids: ids[index], collection });
                        throw new exceptions_1.NotFoundException('No item was found.');
                    }
                    const response = {};
                    response[collection.name] = queryData.Items;
                    return response;
                }));
            })).pipe(operators_1.map(queriesResults => queriesResults.reduce((result, accumulated) => Object.assign(result, accumulated)), {}));
        }
        else {
            this.logger.error('[Batch Get] Collection consistency error', { ids, collections });
            throw new exceptions_1.BadRequestException('Batch input consistency error. Datasets requested must be consistent, all must have indexName or non at all.', { ids, collections });
        }
    }
    find(queries, collection, exclusiveStartKey) {
        let tempQueries = [...queries];
        const scanParams = {
            TableName: collection ? collection.name : models_1.ResourceTable
        };
        this.logger.log('[Find] Prepare for query');
        const orQuieries = tempQueries.filter(query => query.relation === _3a_common_1.Database.DatabaseQueryRelation.Nested);
        tempQueries = tempQueries.filter(query => query.relation !== _3a_common_1.Database.DatabaseQueryRelation.Nested);
        if (tempQueries.length > 0) {
            const expressionAttributeNames = {};
            const expressionAttributeValues = {};
            const filterExpression = tempQueries.map((query, index) => {
                const attributeString = query.key.split('.').map((subkey, subindex) => {
                    expressionAttributeNames[`#attr${index}_${subindex}`] = subkey;
                    return `#attr${index}_${subindex}`;
                }).join('.');
                if (query.value) {
                    expressionAttributeValues[`:attr${index}`] = query.value;
                }
                switch (query.relation) {
                    case _3a_common_1.Database.DatabaseQueryRelation.Equal:
                        return `${attributeString} = :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Exists:
                        return `attribute_exists(${attributeString})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Greater:
                        return `${attributeString} > :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Lesser:
                        return `${attributeString} < :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.NotEqual:
                        return `${attributeString} <> :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.NotExists:
                        return `NOT attribute_exists(${attributeString})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.StartsWith:
                        return `begins_with(${attributeString}, :attr${index})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.In:
                        delete expressionAttributeValues[`:attr${index}`];
                        const inQueries = {};
                        for (let i = 0; i < query.value.length; i++) {
                            const value = query.value[i];
                            inQueries[`:attr${index}_${i}`] = value;
                            expressionAttributeValues[`:attr${index}_${i}`] = value;
                        }
                        return `${attributeString} IN (${Object.keys(inQueries).join(', ')})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Contains:
                        return `contains(${attributeString}, :attr${index})`;
                    default:
                        throw new exceptions_1.BadRequestException('Unkown database query relation: ' + query.relation);
                }
            }).join(' AND ');
            scanParams.ExpressionAttributeNames = expressionAttributeNames;
            scanParams.ExpressionAttributeValues = expressionAttributeValues;
            scanParams.FilterExpression = filterExpression;
            if (collection.index) {
                scanParams.IndexName = collection.index;
            }
            if (exclusiveStartKey) {
                scanParams.ExclusiveStartKey = exclusiveStartKey;
            }
        }
        this.logger.log('[Find] Ready for query');
        return rxjs_1.from(this.dynamo.scan(scanParams).promise().catch(error => {
            this.logger.log('[Find] Internal Error', error);
            throw new exceptions_1.NotFoundException('No item was found in selected table.');
        })).pipe(operators_1.switchMap(scanData => {
            this.logger.log('[Find] Query data returned');
            const finallyCallback = (items) => {
                if (orQuieries.length > 0) {
                    return items.filter(item => {
                        let included = false;
                        for (const query of orQuieries) {
                            if (included) {
                                break;
                            }
                            switch (query.relation) {
                                case database_1.DatabaseQueryRelation.Nested:
                                    if (query.value === undefined) {
                                        this.logger.error('[Find] Invalid query. Nested query must provide a value');
                                        throw new exceptions_1.BadRequestException('[Find] Invalid query. Nested query must provide a value');
                                    }
                                    const keys = query.key.split('.');
                                    let dataFiltered;
                                    while (keys.length > 1) {
                                        dataFiltered = item[keys.shift()];
                                    }
                                    const nestedKey = keys.shift();
                                    if (Array.isArray(dataFiltered)) {
                                        if (Array.isArray(query.value)) {
                                            included = included || query.value
                                                .map(v => (dataFiltered
                                                .findIndex((nestedItem => v === nestedItem[nestedKey] || new RegExp(`.*${v}.*`).test(nestedItem[nestedKey]))) > -1))
                                                .reduce((acc, next) => acc || next, false);
                                        }
                                        else {
                                            included = included ||
                                                (dataFiltered
                                                    .findIndex((nestedItem => query.value === nestedItem[nestedKey] || new RegExp(`.*${query.value}.*`).test(nestedItem[nestedKey]))) > -1);
                                        }
                                    }
                                    else if (Array.isArray(query.value)) {
                                        included = included || query.value
                                            .map(v => dataFiltered[nestedKey] === v)
                                            .reduce((acc, next) => acc || next, false);
                                    }
                                    else {
                                        included = included || query.value === dataFiltered[nestedKey] || new RegExp(`.*${query.value}.*`).test(dataFiltered[nestedKey]);
                                    }
                                    break;
                                default:
                                    break;
                            }
                        }
                        return included;
                    });
                }
                else {
                    return items;
                }
            };
            if (scanData && scanData.LastEvaluatedKey) {
                return this.find(queries, collection, scanData.LastEvaluatedKey).pipe(operators_1.map(items => {
                    return finallyCallback([...scanData.Items, ...items]);
                }));
            }
            if (!scanData || scanData.$response.error || !scanData.Items) {
                this.logger.log('[Find] Not found', { queries: tempQueries, collection });
                throw new exceptions_1.NotFoundException('No item was found.');
            }
            return rxjs_1.of(finallyCallback(scanData.Items));
        }));
    }
    slice(queries, collection, exclusiveStartKey) {
        let tempQueries = [...queries];
        if (tempQueries.length < 2) {
            throw new exceptions_1.BadRequestException('[Slice] This operation requieres a minimum of two queries.');
        }
        if (tempQueries[0].relation !== database_1.DatabaseQueryRelation.Equal) {
            throw new exceptions_1.BadRequestException('[Slice] This operation requieres a partition equality query.');
        }
        const partitionFilter = tempQueries.shift();
        const rangeKey = tempQueries[0].key;
        const rangeFilter1 = tempQueries.shift();
        const rangeFilter2 = tempQueries.shift();
        const queryParams = {
            TableName: collection ? collection.name : models_1.ResourceTable,
            ExpressionAttributeNames: {
                '#partition': partitionFilter.key,
                '#range': rangeKey
            },
            ExpressionAttributeValues: {
                ':partition': partitionFilter.value,
                ':range1': rangeFilter1.value,
            }
        };
        if (exclusiveStartKey) {
            queryParams.ExclusiveStartKey = exclusiveStartKey;
        }
        if (collection.index) {
            queryParams.IndexName = collection.index;
        }
        if (rangeFilter2) {
            queryParams.ExpressionAttributeValues[':range2'] = rangeFilter2.value;
            queryParams.KeyConditionExpression = '#partition = :partition AND (#range BETWEEN :range1 AND :range2)';
        }
        else {
            let sortQuery;
            switch (rangeFilter1.relation) {
                case _3a_common_1.Database.DatabaseQueryRelation.Equal:
                    sortQuery = '#range = :range1';
                    break;
                case _3a_common_1.Database.DatabaseQueryRelation.Contains:
                    sortQuery = 'contains(#range, :range1)';
                    break;
                case _3a_common_1.Database.DatabaseQueryRelation.StartsWith:
                    sortQuery = 'begins_with(#range, :range1)';
                    break;
                default:
                    throw new exceptions_1.BadRequestException('[Slice] This operation requieres a sort-key equality, contains, or starts-wit query.');
            }
            queryParams.KeyConditionExpression = `#partition = :partition AND ${sortQuery}`;
        }
        tempQueries = tempQueries.filter(query => query.key !== rangeKey);
        const orQuieries = tempQueries.filter(query => query.relation === _3a_common_1.Database.DatabaseQueryRelation.Nested);
        tempQueries = tempQueries.filter(query => query.relation !== _3a_common_1.Database.DatabaseQueryRelation.Nested);
        this.logger.log('[Find] Prepare for query');
        if (tempQueries.length > 0) {
            const filterExpression = tempQueries.map((query, index) => {
                const attributeString = query.key.split('.').map((subkey, subindex) => {
                    queryParams.ExpressionAttributeNames[`#attr${index}_${subindex}`] = subkey;
                    return `#attr${index}_${subindex}`;
                }).join('.');
                if (query.value) {
                    queryParams.ExpressionAttributeValues[`:attr${index}`] = query.value;
                }
                switch (query.relation) {
                    case _3a_common_1.Database.DatabaseQueryRelation.Equal:
                        return `${attributeString} = :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Exists:
                        return `attribute_exists(${attributeString})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Greater:
                        return `${attributeString} > :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Lesser:
                        return `${attributeString} < :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.NotEqual:
                        return `${attributeString} <> :attr${index}`;
                    case _3a_common_1.Database.DatabaseQueryRelation.NotExists:
                        return `NOT attribute_exists(${attributeString})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.StartsWith:
                        return `begins_with(${attributeString}, :attr${index})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.In:
                        delete queryParams.ExpressionAttributeValues[`:attr${index}`];
                        const inQueries = {};
                        for (let i = 0; i < query.value.length; i++) {
                            const value = query.value[i];
                            inQueries[`:attr${index}_${i}`] = value;
                            queryParams.ExpressionAttributeValues[`:attr${index}_${i}`] = value;
                        }
                        return `${attributeString} IN (${Object.keys(inQueries).join(', ')})`;
                    case _3a_common_1.Database.DatabaseQueryRelation.Contains:
                        return `contains(${attributeString}, :attr${index})`;
                    default:
                        throw new exceptions_1.BadRequestException('Unkown database query relation: ' + query.relation);
                }
            }).join(' AND ');
            queryParams.FilterExpression = filterExpression;
        }
        this.logger.log('[Find] Ready for query');
        return rxjs_1.from(this.dynamo.query(queryParams).promise().catch(error => {
            this.logger.log('[Slice] Internal Error', error);
            throw new exceptions_1.NotFoundException('No item was found in selected table.');
        })).pipe(operators_1.switchMap(sliceData => {
            this.logger.log('[Slice] Slice data returned');
            const finallyCallback = (items) => {
                if (orQuieries.length > 0) {
                    return items.filter(item => {
                        let included = false;
                        for (const query of orQuieries) {
                            if (included) {
                                break;
                            }
                            switch (query.relation) {
                                case database_1.DatabaseQueryRelation.Nested:
                                    if (query.value === undefined) {
                                        this.logger.error('[Find] Invalid query. Nested query must provide a value');
                                        throw new exceptions_1.BadRequestException('[Find] Invalid query. Nested query must provide a value');
                                    }
                                    const keys = query.key.split('.');
                                    let dataFiltered;
                                    while (keys.length > 1) {
                                        dataFiltered = item[keys.shift()];
                                    }
                                    const nestedKey = keys.shift();
                                    if (Array.isArray(dataFiltered)) {
                                        if (Array.isArray(query.value)) {
                                            included = included || query.value
                                                .map(v => (dataFiltered
                                                .findIndex((nestedItem => v === nestedItem[nestedKey] || new RegExp(`.*${v}.*`).test(nestedItem[nestedKey]))) > -1))
                                                .reduce((acc, next) => acc || next, false);
                                        }
                                        else {
                                            included = included ||
                                                (dataFiltered
                                                    .findIndex((nestedItem => query.value === nestedItem[nestedKey] || new RegExp(`.*${query.value}.*`).test(nestedItem[nestedKey]))) > -1);
                                        }
                                    }
                                    else if (Array.isArray(query.value)) {
                                        included = included || query.value
                                            .map(v => dataFiltered[nestedKey] === v)
                                            .reduce((acc, next) => acc || next, false);
                                    }
                                    else {
                                        included = included || query.value === dataFiltered[nestedKey] || new RegExp(`.*${query.value}.*`).test(dataFiltered[nestedKey]);
                                    }
                                    break;
                                default:
                                    break;
                            }
                        }
                        return included;
                    });
                }
                else {
                    return items;
                }
            };
            if (sliceData && sliceData.LastEvaluatedKey) {
                return this.slice(queries, collection, sliceData.LastEvaluatedKey).pipe(operators_1.map(items => {
                    return finallyCallback([...sliceData.Items, ...items]);
                }));
            }
            if (!sliceData || sliceData.$response.error || !sliceData.Items) {
                this.logger.log('[Slice] Not found', { queries: tempQueries, collection });
                throw new exceptions_1.NotFoundException('No item was found.');
            }
            return rxjs_1.of(finallyCallback(sliceData.Items));
        }));
    }
};
DynamoService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('aws')),
    __param(1, common_1.Inject('loggerEnabled')), __param(1, common_1.Optional()),
    __metadata("design:paramtypes", [Object, Boolean])
], DynamoService);
exports.DynamoService = DynamoService;
