"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var $3AModule_1;
const AWS = require("aws-sdk");
const CognitoExpress = require("cognito-express");
const common_1 = require("@nestjs/common");
const resource_service_1 = require("./services/resource.service");
const resource_controller_1 = require("./controllers/resource.controller");
const dynamo_service_1 = require("./database/dynamo.service");
const navigation_service_1 = require("./services/navigation.service");
const middleware_1 = require("./middleware");
const proxy_service_1 = require("./services/proxy.service");
const database_controller_1 = require("./controllers/database.controller");
const proxy_controller_1 = require("./controllers/proxy.controller");
const database_service_1 = require("./services/database.service");
const exceptions_1 = require("./exceptions");
const notification_service_1 = require("./services/notification.service");
const notification_controller_1 = require("./controllers/notification.controller");
const security_service_1 = require("./services/security.service");
const users_controller_1 = require("./controllers/users.controller");
const users_service_1 = require("./services/users.service");
const report_service_1 = require("./services/report.service");
const report_controller_1 = require("./controllers/report.controller");
let $3AModule = $3AModule_1 = class $3AModule {
    static useAWS(awsConfig, appConfig, useLogger = false) {
        AWS.config.update({
            accessKeyId: awsConfig.accessKeyId,
            secretAccessKey: awsConfig.secretAccessKey,
            region: awsConfig.region
        });
        return {
            module: $3AModule_1,
            providers: [
                {
                    provide: 'loggerEnabled',
                    useValue: useLogger
                },
                {
                    provide: 'app',
                    useValue: appConfig
                },
                {
                    provide: 'aws',
                    useValue: AWS
                }, {
                    provide: 'database',
                    useClass: dynamo_service_1.DynamoService
                }, {
                    provide: 'cognito',
                    useValue: new CognitoExpress({
                        region: awsConfig.region,
                        cognitoUserPoolId: awsConfig.cognitoUserPoolId,
                        tokenUse: awsConfig.tokenUse || 'access',
                        tokenExpiration: awsConfig.tokenExpiration || 3600000
                    })
                }
            ],
            exports: [
                {
                    provide: 'database',
                    useClass: dynamo_service_1.DynamoService
                },
                notification_service_1.NotificationService,
            ]
        };
    }
    configure(consumer) {
        consumer
            .apply(middleware_1.I3AMAuthenticatedMiddleware)
            .with(false)
            .forRoutes(resource_controller_1.ResourceController, database_controller_1.DatabaseController)
            .apply(middleware_1.I3AMAuthenticatedMiddleware)
            .with(true)
            .forRoutes(users_controller_1.UsersController);
    }
};
$3AModule = $3AModule_1 = __decorate([
    common_1.Module({
        imports: [common_1.HttpModule],
        controllers: [resource_controller_1.ResourceController, database_controller_1.DatabaseController, proxy_controller_1.ProxyController, notification_controller_1.NotificationController,
            users_controller_1.UsersController, report_controller_1.ReportController],
        providers: [
            resource_service_1.ResourceService,
            navigation_service_1.ResourceNavigationService,
            database_service_1.ServerDatabaseService,
            security_service_1.SecurityService,
            notification_service_1.NotificationService,
            report_service_1.ReportService,
            users_service_1.UserService,
            proxy_service_1.ProxyService,
            exceptions_1.ErrorFilter,
            exceptions_1.ErrorsInterceptor,
        ],
        exports: [
            resource_service_1.ResourceService,
            navigation_service_1.ResourceNavigationService,
            database_service_1.ServerDatabaseService,
            security_service_1.SecurityService,
            notification_service_1.NotificationService,
            users_service_1.UserService,
            proxy_service_1.ProxyService,
            exceptions_1.ErrorFilter,
            exceptions_1.ErrorsInterceptor,
        ],
    })
], $3AModule);
exports.$3AModule = $3AModule;
