"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const operators_1 = require("rxjs/operators");
const rxjs_1 = require("rxjs");
function permissionsCheck(resourceID, accessLevel, permissions) {
    if (!resourceID) {
        return false;
    }
    const IAMAppResource = resourceID.split('::')[0];
    const IAMInternalResource = resourceID.split('::')[1];
    for (const permission of permissions) {
        if (!permission.resourceID) {
            continue;
        }
        const permissionApp = permission.resourceID.split('::')[0];
        const permissionResource = permission.resourceID.split('::')[1];
        if (matchWildcards(IAMAppResource, permissionApp)) {
            if (matchWildcards(IAMInternalResource, permissionResource) && permission.access[accessLevel]) {
                return true;
            }
        }
    }
    return false;
}
exports.permissionsCheck = permissionsCheck;
function matchWildcards(str, rule) {
    if (!rule) {
        return false;
    }
    return new RegExp('.*' + rule.split('*').join('.*') + '.*').test(str);
}
exports.matchWildcards = matchWildcards;
function CanUserPerformAction(db, resourceIDs, user, action) {
    const allUserPermissions = user.permissions ? user.permissions.slice() : [];
    if (user.roles && user.roles.length > 0) {
        return db.batchGet([user.roles], [{ name: 'Roles' }]).pipe(operators_1.map(result => {
            const roles = result.Roles;
            for (const role of roles) {
                allUserPermissions.push(...role.permissions);
            }
            if (typeof resourceIDs === 'string') {
                return permissionsCheck(resourceIDs, action, allUserPermissions);
            }
            else {
                const innerResult = [];
                for (const resourceId of resourceIDs) {
                    if (innerResult.indexOf(resourceId) < 0 && permissionsCheck(resourceId, action, allUserPermissions)) {
                        innerResult.push(resourceId);
                    }
                }
                return innerResult;
            }
        }, operators_1.catchError(() => rxjs_1.of([]))));
    }
    else {
        if (typeof resourceIDs === 'string') {
            return rxjs_1.of(permissionsCheck(resourceIDs, action, allUserPermissions));
        }
        else {
            const result = [];
            for (const resourceId of resourceIDs) {
                if (result.indexOf(resourceId) < 0 && permissionsCheck(resourceId, action, allUserPermissions)) {
                    result.push(resourceId);
                }
            }
            return rxjs_1.of(result);
        }
    }
}
exports.CanUserPerformAction = CanUserPerformAction;
function withPermissionsCanPerformAction(resourceIDs, permissions, action) {
    if (typeof resourceIDs === 'string') {
        return permissionsCheck(resourceIDs, action, permissions);
    }
    else {
        const result = [];
        for (const resourceId of resourceIDs) {
            if (result.indexOf(resourceId) < 0 && permissionsCheck(resourceId, action, permissions)) {
                result.push(resourceId);
            }
        }
        return result;
    }
}
exports.withPermissionsCanPerformAction = withPermissionsCanPerformAction;
