"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const _3a_common_1 = require("3a-common");
const operators_1 = require("rxjs/operators");
const rxjs_1 = require("rxjs");
const models_1 = require("../models");
let ResourceNavigationService = class ResourceNavigationService {
    constructor(db) {
        this.db = db;
    }
    matchWildcards(str, rule) {
        return new RegExp('^' + rule.split('*').join('.*') + '$').test(str);
    }
    permissionsCheck(resourceID, accessLevel, permissions) {
        if (!resourceID) {
            return false;
        }
        const IAMAppResource = resourceID.split('::')[0];
        const IAMInternalResource = resourceID.split('::')[1];
        for (const permission of permissions) {
            const permissionApp = permission.resourceID.split('::')[0];
            const permissionResource = permission.resourceID.split('::')[1];
            if (this.matchWildcards(IAMAppResource, permissionApp)) {
                if (this.matchWildcards(IAMInternalResource, permissionResource)) {
                    return permission.access[accessLevel];
                }
            }
        }
        return false;
    }
    getNavigationTree(user, module) {
        if (!user) {
            return rxjs_1.of(undefined);
        }
        return this.flatUserPermissions(user).pipe(operators_1.switchMap(permissions => {
            const tree = this.createNavigationTree(permissions, module);
            if (tree.children.length === 1 && tree.children[0].resourceID === '*') {
                const access = tree.children[0].access;
                return this.db.find([{
                        key: 'id',
                        relation: _3a_common_1.Database.DatabaseQueryRelation.NotEqual,
                        value: '*'
                    }], { name: models_1.OrganizationsTable }).pipe(operators_1.map(organizations => {
                    tree.children = organizations.map(organization => {
                        return {
                            resourceID: organization.id,
                            type: -1,
                            children: (organization.resources || [])
                                .map(resourceID => {
                                return {
                                    resourceID,
                                    children: [],
                                    access
                                };
                            })
                        };
                    });
                    return tree;
                }));
            }
            else {
                return rxjs_1.of(tree);
            }
        }));
    }
    flatUserPermissions(user) {
        if (!user) {
            return rxjs_1.of([]);
        }
        const userFlatPermissions = user.permissions || [];
        if (user.roles && user.roles.length > 0) {
            return this.db.batchGet([user.roles], [{ name: 'Roles' }]).pipe(operators_1.map(responses => {
                const rolesFlatPermissions = responses.Roles
                    .map(role => role.permissions || [])
                    .reduce((flatPermissions, rolePermissions) => {
                    flatPermissions.push(...rolePermissions);
                    return flatPermissions;
                }, []);
                userFlatPermissions.push(...rolesFlatPermissions);
                return userFlatPermissions;
            }));
        }
        else {
            return rxjs_1.of(userFlatPermissions);
        }
    }
    createNavigationTree(permissions, module) {
        const filteredPermissions = permissions.filter(permission => {
            return permission.resourceID && this.matchWildcards(module, permission.resourceID.split('::')[0]);
        }).map(permission => {
            permission.resourceID = permission.resourceID.split('::')[1];
            return permission;
        });
        const resourceGraph = {
            resourceID: '',
            children: []
        };
        for (const permission of filteredPermissions) {
            const permissionAccesLevels = Object.keys(permission.access);
            let currentNode = resourceGraph;
            const permissionSegments = permission.resourceID.split(':');
            let nodeID = permissionSegments.shift();
            while (nodeID) {
                const currentNodeChildrenFiltered = currentNode.children.map((nodeChild, index) => {
                    return {
                        index,
                        value: nodeChild
                    };
                }).filter(nodeChild => {
                    return nodeChild.value.resourceID === nodeID ||
                        nodeChild.value.resourceID === '*' ||
                        nodeID === '*';
                });
                if (permissionSegments.length === 0) {
                    if (currentNodeChildrenFiltered.length > 0) {
                        const currentNodeChildMatch = currentNodeChildrenFiltered[0].value;
                        currentNodeChildMatch.resourceID = nodeID === '*' ? nodeID : currentNodeChildMatch.resourceID;
                        if (currentNodeChildMatch.access) {
                            for (const accessLevel of permissionAccesLevels) {
                                currentNodeChildMatch.access[accessLevel] = currentNodeChildMatch.access[accessLevel] ||
                                    permission.access[accessLevel];
                            }
                        }
                        else {
                            currentNodeChildMatch.access = permission.access;
                        }
                        currentNode.children[currentNodeChildrenFiltered[0].index] = currentNodeChildMatch;
                    }
                    else {
                        currentNode.children.push({
                            resourceID: nodeID,
                            access: permission.access,
                            type: permission.type,
                            children: []
                        });
                    }
                    break;
                }
                else {
                    if (currentNodeChildrenFiltered.length > 0) {
                        const currentNodeChildMatch = currentNodeChildrenFiltered[0].value;
                        const accessAllowedOverridesPermission = permissionAccesLevels.map(accessLevel => {
                            return (currentNodeChildMatch.access &&
                                (currentNodeChildMatch.access[accessLevel]) || !permission.access[accessLevel]);
                        }).reduce((accumulated, current) => accumulated && current, true);
                        if (accessAllowedOverridesPermission) {
                            break;
                        }
                        else {
                            currentNode = currentNode.children[currentNodeChildrenFiltered[0].index];
                            nodeID = permissionSegments.shift();
                        }
                    }
                    else {
                        const branch = {
                            resourceID: nodeID,
                            children: []
                        };
                        let currentBranchLevel = branch;
                        let segment = permissionSegments.shift();
                        while (permissionSegments.length > 0) {
                            currentBranchLevel.children.push({
                                resourceID: segment,
                                children: []
                            });
                            currentBranchLevel = currentBranchLevel.children[0];
                            segment = permissionSegments.shift();
                        }
                        currentBranchLevel.children.push({
                            resourceID: segment,
                            access: permission.access,
                            type: permission.type,
                            children: []
                        });
                        resourceGraph.children.push(branch);
                        break;
                    }
                }
            }
        }
        return this.reduceNavigationTree(resourceGraph);
    }
    reduceNavigationTree(graph) {
        const tree = {
            resourceID: '',
            children: []
        };
        for (const graphBranch of graph.children) {
            const directBranch = this.directAccessChild(graphBranch);
            tree.children.push(...directBranch);
        }
        return tree;
    }
    directAccessChild(graph) {
        if (graph.access) {
            return [graph];
        }
        else {
            const directChilds = [];
            for (const child of graph.children) {
                directChilds.push(...this.directAccessChild(child));
            }
            return directChilds;
        }
    }
};
ResourceNavigationService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('database')),
    __metadata("design:paramtypes", [_3a_common_1.Database.DatabaseService])
], ResourceNavigationService);
exports.ResourceNavigationService = ResourceNavigationService;
