"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const MailComposer = require('nodemailer/lib/mail-composer');
const rxjs_1 = require("rxjs");
const logger_service_1 = require("./logger.service");
let NotificationService = class NotificationService {
    constructor(AWS, appConfig, loggerEnabled) {
        this.AWS = AWS;
        this.appConfig = appConfig;
        this.ses = new this.AWS.SES({
            apiVersion: '2010-12-01',
            region: 'us-east-1',
        });
        this.logger = new logger_service_1.Logger('Notification Service', loggerEnabled);
        this.setupGlobalExceptionNotifications();
    }
    sendEmail(email, subject, text, attachment, html) {
        const mailOptions = {
            from: this.appConfig.emailSender ||
                'Tres Astronautas <no-reply@tresastronautas.com>',
            sender: this.appConfig.emailSender || 'no-reply@tresastronautas.com',
            to: email,
            subject,
            text,
            attachments: attachment !== undefined
                ? [
                    {
                        filename: attachment.filename,
                        path: attachment.path,
                    },
                ]
                : [],
        };
        if (html) {
            mailOptions.html = html;
        }
        const mail = new MailComposer(mailOptions).compile();
        return rxjs_1.from(new Promise((resolve, reject) => {
            mail.build((err, message) => {
                this.ses.sendRawEmail({ RawMessage: { Data: message } }, (error, data) => {
                    if (error) {
                        console.log(error);
                        resolve({ success: false, message: error });
                    }
                    else {
                        console.log(data);
                        resolve({ success: true, message: data.messageId });
                        this.logger.log('Message sent: %s', data);
                    }
                });
            });
        }));
    }
    setupGlobalExceptionNotifications() {
        process.on('uncaughtException', err => {
            console.error('Application crashed.', err.message);
            console.error(err.stack);
            this.sendEmail('developer@tresastronautas.com', `[${this.appConfig.name}] Application crashed`, `Application has crashed
          ${err.name}: ${err.message}
          Trace Log:
          ${err.stack};`).subscribe();
            process.exit(-1);
        });
    }
};
NotificationService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('aws')),
    __param(1, common_1.Inject('app')),
    __param(2, common_1.Inject('loggerEnabled')),
    __param(2, common_1.Optional()),
    __metadata("design:paramtypes", [Object, Object, Boolean])
], NotificationService);
exports.NotificationService = NotificationService;
