"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
const logger_service_1 = require("./logger.service");
const htmlPdf = require("html-pdf-chrome");
const pug = require("pug");
const moment = require("moment");
const notification_service_1 = require("./notification.service");
const OPTIONS = {
    port: 9222,
    printOptions: {
        marginLeft: 0,
        marginRight: 0
    }
};
let ReportService = class ReportService {
    constructor(AWS, appConfig, loggerEnabled, serviceNotification) {
        this.AWS = AWS;
        this.appConfig = appConfig;
        this.serviceNotification = serviceNotification;
        this.ses = new this.AWS.SES({
            apiVersion: '2010-12-01',
            region: 'us-east-1'
        });
        this.logger = new logger_service_1.Logger('Report Service', loggerEnabled);
    }
    downloadReport(attachment, params) {
        return rxjs_1.from(new Promise((resolve, reject) => {
            const array = params.prescriptionType;
            const template = pug.compileFile(attachment.path);
            const html2 = template(params);
            const fileName = `temp/file${params.fileName}__${moment()
                .format('YYYY_MM_DD_HH_mm_ss')}.pdf`;
            htmlPdf.create(html2, OPTIONS).then((pdf) => {
                pdf.toFile(fileName).then(_ => {
                    resolve({ success: true, message: fileName });
                }).catch(_ => {
                    resolve({ success: false, message: 'Reports Module: Sensor report could not be sent.' });
                });
            }).catch(_ => {
                resolve({ success: false, message: 'Reports Module: Rendered could not load HTML.' });
            });
        }));
    }
    createReport(email, subject, text, html, attachment, params) {
        return this.generatePDF(email, subject, text, html, params, attachment);
    }
    generatePDF(email, subject, text, html, params, templatePath) {
        return rxjs_1.from(new Promise((resolve, reject) => {
            const array = params.array;
            array.forEach(element => {
                const template = pug.compileFile(templatePath.path);
                const pa = params;
                pa.array = element;
                const html2 = template(pa);
                const fileName = `temp/file${params.fileName}_${element}_${moment()
                    .format('YYYY_MM_DD_HH_mm_ss')}.pdf`;
                htmlPdf.create(html2, OPTIONS).then((pdf) => {
                    pdf.toFile(fileName).then(_ => {
                        if (email !== undefined) {
                            this.serviceNotification.sendEmail(email, subject, text, { path: fileName,
                                filename: `file${params.fileName}_${element}_${moment()
                                    .format('YYYY_MM_DD_HH_mm_ss')}.pdf` }, html);
                        }
                    }).catch(_ => {
                        resolve({ success: false, message: 'PDF could not be sent.' });
                    });
                }).catch(_ => {
                    resolve({ success: false, message: 'Reports Module: Rendered could not load HTML.' });
                });
            });
            resolve({ success: true, message: 'PDF sent.' });
        }));
    }
};
ReportService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('aws')),
    __param(1, common_1.Inject('app')),
    __param(2, common_1.Inject('loggerEnabled')), __param(2, common_1.Optional()),
    __metadata("design:paramtypes", [Object, Object, Boolean, notification_service_1.NotificationService])
], ReportService);
exports.ReportService = ReportService;
