"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const _3a_common_1 = require("3a-common");
const rxjs_1 = require("rxjs");
const models_1 = require("../models");
const operators_1 = require("rxjs/operators");
const uuid = require("uniqid");
const exceptions_1 = require("../exceptions");
const navigation_service_1 = require("./navigation.service");
const methods_1 = require("../models/methods");
let ResourceService = class ResourceService {
    constructor(db, appConfig, navigationService) {
        this.db = db;
        this.appConfig = appConfig;
        this.navigationService = navigationService;
    }
    getAll(user) {
        return this.navigationService.getNavigationTree(user, `3a:${this.appConfig.name}:resources`);
    }
    get(id, user, collection, attribute) {
        if (attribute) {
            return this.db.get(id, collection, attribute);
        }
        return this.db.get(id, { name: models_1.ResourceTable }).pipe(operators_1.switchMap(resource => {
            if (resource && user.roles && user.roles.length > 0) {
                return this.db.batchGet([user.roles], [{ name: 'Roles' }]).pipe(operators_1.switchMap(result => {
                    const allUserPermissions = user.permissions ? user.permissions.slice() : [];
                    for (const role of result.Roles) {
                        allUserPermissions.push(...role.permissions);
                    }
                    if (methods_1.permissionsCheck(resource.resourceID, 'read', allUserPermissions)) {
                        if (resource.tableName) {
                            return this.db.get(id, {
                                name: resource.tableName,
                                index: (collection && collection.index) ? collection.index : undefined
                            });
                        }
                        else {
                            return rxjs_1.of(resource);
                        }
                    }
                    else {
                        throw new exceptions_1.UnauthorizedException(`El usuario no puede leer el recurso (${resource.resourceID})`);
                    }
                }));
            }
            else {
                if (resource) {
                    if (resource.resourceID.includes(user.id)
                        || methods_1.permissionsCheck(resource.resourceID, 'read', user.permissions ? user.permissions : [])) {
                        if (resource.tableName) {
                            return this.db.get(id, {
                                name: resource.tableName,
                                index: (collection && collection.index) ? collection.index : undefined
                            });
                        }
                        else {
                            return rxjs_1.of(resource);
                        }
                    }
                    else {
                        throw new exceptions_1.UnauthorizedException(`El usuario no puede leer el recurso (${resource.resourceID})`);
                    }
                }
                else {
                    return this.db.get(id, collection);
                }
            }
        }));
    }
    save(data, user, merge = true, parentResourceID = `3a:${this.appConfig.name}:resources:`) {
        if (!data.id) {
            data.id = uuid();
            data.createdAt = new Date().getTime();
            if (data.partitionID && data.partitionID.endsWith('-')) {
                data.partitionID = `${data.partitionID}${data.id}`;
            }
            if (data.sortID && data.sortID.endsWith('-')) {
                data.sortID = `${data.sortID}${data.id}`;
            }
        }
        data.lastModified = new Date().getTime();
        if (data.resourceID && data.resourceID.endsWith(':')) {
            data.resourceID = `${data.resourceID}${data.id}`;
        }
        const resourceItem = {
            id: data.id,
            resourceID: data.resourceID || `${parentResourceID}:${data.id}`,
            name: data.name
        };
        if (!resourceItem.id || !resourceItem.name || !resourceItem.resourceID) {
            throw new exceptions_1.BadRequestException('Resource needs to have \'name\' and \'resourceID\'', data);
        }
        data.resourceID = resourceItem.resourceID;
        if (data.disabled) {
            resourceItem.disabled = data.disabled;
        }
        if (data.tableName) {
            resourceItem.tableName = data.tableName;
        }
        if (data.type) {
            resourceItem.type = data.type;
        }
        if (user) {
            return methods_1.CanUserPerformAction(this.db, data.resourceID, user, 'write').pipe(operators_1.switchMap(isUserAllowed => {
                if (isUserAllowed) {
                    return this.db.save(resourceItem, { name: models_1.ResourceTable }, merge).pipe(operators_1.switchMap(resource => {
                        if (data.tableName || resource.tableName) {
                            return this.db.save(data, { name: data.tableName || resource.tableName }, merge);
                        }
                        else {
                            return rxjs_1.of(resource);
                        }
                    }));
                }
                else {
                    throw new exceptions_1.UnauthorizedException(`El usuario no puede escribir el recurso (${data.resourceID})`);
                }
            }));
        }
        else {
            return this.db.save(resourceItem, { name: models_1.ResourceTable }, merge).pipe(operators_1.switchMap(resource => {
                if (data.tableName || resource.tableName) {
                    return this.db.save(data, { name: data.tableName || resource.tableName }, merge);
                }
                else {
                    return rxjs_1.of(resource);
                }
            }));
        }
    }
};
ResourceService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject('database')),
    __param(1, common_1.Inject('app')),
    __metadata("design:paramtypes", [_3a_common_1.Database.DatabaseService, Object, navigation_service_1.ResourceNavigationService])
], ResourceService);
exports.ResourceService = ResourceService;
