"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
const _3a_common_1 = require("3a-common");
const operators_1 = require("rxjs/operators");
const AWS = require("aws-sdk");
const logger_service_1 = require("./logger.service");
const notification_service_1 = require("./notification.service");
const security_service_1 = require("./security.service");
const exceptions_1 = require("../exceptions");
let UserService = class UserService {
    constructor(notificationSvc, security, appConfig, database, loggerEnabled) {
        this.notificationSvc = notificationSvc;
        this.security = security;
        this.appConfig = appConfig;
        this.database = database;
        this.logger = new logger_service_1.Logger('User Service', loggerEnabled);
    }
    invite(user) {
        return this.notificationSvc.sendEmail(user.email, '[Invitación] Bienvenido!', '', undefined, `
        <div>
        Para completar tu registro haz click
        <a href="${this.appConfig.url}/login/complete/${this.security.encrypt(user.email)}">acá</a></div>
        `);
    }
    uncompleted(token) {
        this.logger.log(token, 'Uncompleted');
        const email = this.security.decrypt(token);
        this.logger.log(email, 'Uncompleted');
        return this.database
            .find([
            {
                key: 'email',
                relation: _3a_common_1.Database.DatabaseQueryRelation.Equal,
                value: email,
            },
        ], { name: 'Users' })
            .pipe(operators_1.map(users => {
            if (users && users[0]) {
                return users[0];
            }
            else {
                return undefined;
            }
        }));
    }
    confirm(userID, user, token) {
        if (userID === undefined || userID === '') {
            throw new exceptions_1.BadRequestException("You must send user ID in query param 'id' for user lookup.");
        }
        if (user.id === undefined || user.id === '') {
            throw new exceptions_1.BadRequestException("User sent is missing 'id' for user recreation.");
        }
        if (user.email === undefined || user.email === '') {
            throw new exceptions_1.BadRequestException("User sent is missing 'email' for user recreation.");
        }
        if (user.email !== this.security.decrypt(token)) {
            throw new exceptions_1.UnauthorizedException('You do not have the permissions to perform this request.');
        }
        return this.database
            .find([
            {
                key: 'email',
                relation: _3a_common_1.Database.DatabaseQueryRelation.Equal,
                value: user.email,
            },
        ], { name: 'Users' })
            .pipe(operators_1.map(users => users.find(u => u.id === user.id)), operators_1.switchMap(loadedUser => {
            new AWS.DynamoDB.DocumentClient({ apiVersion: '2012-08-10' }).delete({
                TableName: 'Resources',
                Key: {
                    id: loadedUser.id,
                },
            });
            const params = {
                TableName: 'Users',
                Key: {
                    id: loadedUser.id,
                },
            };
            return rxjs_1.from(new AWS.DynamoDB.DocumentClient({ apiVersion: '2012-08-10' })
                .delete(params)
                .promise()).pipe(operators_1.switchMap(err => {
                if (err.$response.error) {
                    this.logger.error('Unable to remove pending user creation info.');
                    throw new Error('Unable to remove pending user creation info.');
                }
                else {
                    this.logger.log('Pending creation removed.');
                    user = Object.assign(user, { id: userID });
                    if (!user.roles) {
                        user = Object.assign(user, { roles: [] });
                    }
                    if (!user.permissions) {
                        user = Object.assign(user, { permissions: [] });
                    }
                    this.database
                        .save({
                        id: user.id,
                        name: user.name,
                        tableName: 'Users',
                        resourceID: user.resourceID,
                    }, { name: 'Resources' })
                        .subscribe();
                    return this.database.save(user, { name: 'Users' }).pipe(operators_1.switchMap(savedUser => {
                        return this.database
                            .get(savedUser['nit'] || savedUser['organizations'][0], {
                            name: 'Organizations',
                        })
                            .pipe(operators_1.switchMap(organization => {
                            organization['users'] = organization['users'] || [];
                            const index = organization['users'].indexOf(loadedUser.id);
                            if (index >= 0 || user['migrated']) {
                                if (index >= 0) {
                                    organization['users'].splice(index, 1, user.id);
                                }
                                else {
                                    organization['users'].push(user.id);
                                }
                                return this.database.save(organization, {
                                    name: 'Organizations',
                                });
                            }
                            else {
                                throw new exceptions_1.BadRequestException('User is member of an organization which has not added its email.');
                            }
                        }));
                    }));
                }
            }));
        }));
    }
    recover(email) {
        if (email === undefined || email === '') {
            throw new exceptions_1.BadRequestException("User sent is missing 'email' for user recreation.");
        }
        return this.notificationSvc.sendEmail(email, '[Recuperación] Recupera tu cuenta', '', undefined, `
        <div>
        El administrador de tu cuenta nos ha pedido que te enviemos un correo de recuperación.
        Para recuperar tu cuenta haz click
        <a href="${this.appConfig.url}login/recover/${email}">acá</a>

        Si no reconoces esta actividad, comunícate con nosotros, o cambia tu contraseña ahora.
        </div>
        `);
    }
};
UserService = __decorate([
    common_1.Injectable(),
    __param(2, common_1.Inject('app')),
    __param(3, common_1.Inject('database')),
    __param(4, common_1.Inject('loggerEnabled')),
    __param(4, common_1.Optional()),
    __metadata("design:paramtypes", [notification_service_1.NotificationService,
        security_service_1.SecurityService, Object, _3a_common_1.Database.DatabaseService, Boolean])
], UserService);
exports.UserService = UserService;
